@@ -17,9 +17,9 @@ pub trait Approximate<Destination> {
 
 #[derive(Clone, Debug, PartialEq, Eq, PartialOrd, Ord)]
 pub enum Similarity {
-    Equivalent,
-    Subequal,
     Different,
+    Subequal,
+    Equivalent,
 }
 
 use Similarity::*;
@@ -268,43 +268,29 @@ impl Approximate<types::Type> for Type {
                     ..
                 },
             ) => {
-                // FIXME: This block is hard to read.
                 let mut sims = q.approx(i, generics, substs);
                 if sims == vec![Equivalent] {
                     match (q_args, i_args) {
-                        (Some(q), Some(i)) => match **i {
-                            types::GenericArgs::AngleBracketed { args: ref i, .. } => {
-                                let GenericArgs::AngleBracketed { args: ref q } = **q;
-                                for (i, q) in i
-                                    .iter()
-                                    .filter_map(|i| match i {
-                                        types::GenericArg::Type(types::Type::Generic(i)) => Some(i),
-                                        _ => None,
-                                    })
-                                    .zip(q.iter().map(|q| match q {
-                                        GenericArg::Type(q) => q,
-                                    }))
-                                {
-                                    match substs.get(i) {
-                                        Some(i) => {
-                                            if q == i {
-                                                sims.push(Subequal)
-                                            } else {
-                                                sims.push(Different)
-                                            }
-                                        }
-                                        None => {
-                                            substs.insert(i.clone(), q.clone());
-                                            sims.push(Subequal)
-                                        }
-                                    }
+                        (Some(q), Some(i)) => {
+                            if let (
+                                GenericArgs::AngleBracketed { args: q },
+                                types::GenericArgs::AngleBracketed { args: i, .. },
+                            ) = (&**q, &**i)
+                            {
+                                let q = q.iter().map(|q| match q {
+                                    GenericArg::Type(q) => q,
+                                });
+                                let i = i.iter().filter_map(|i| match i {
+                                    types::GenericArg::Type(t) => Some(t),
+                                    _ => None,
+                                });
+                                for (q, i) in q.zip(i) {
+                                    sims.append(&mut q.approx(i, generics, substs))
                                 }
                             }
-                            types::GenericArgs::Parenthesized { .. } => {}
-                        },
+                        }
                         (Some(_), None) => sims.push(Different),
-                        (None, Some(_)) => {}
-                        (None, None) => {}
+                        (None, _) => {}
                     }
                 }
                 sims
